#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for Requests."""

from requests import Response
from os import environ
from txrequests import Session
from twisted.trial.unittest import TestCase
from twisted.internet import defer

HTTPBIN = environ.get('HTTPBIN_URL', 'http://httpbin.org/')

def httpbin(*suffix):
    """Returns url for HTTPBIN resource."""
    return HTTPBIN + '/'.join(suffix)


class RequestsTestCase(TestCase):

    @defer.inlineCallbacks
    def test_session(self):
        # basic futures get
        with Session() as sess:
            d = sess.get(httpbin('get'))
            self.assertIsInstance(d, defer.Deferred)
            resp = yield d
            self.assertIsInstance(resp, Response)
            self.assertEqual(200, resp.status_code)

            # non-200, 404
            d = sess.get(httpbin('status/404'))
            resp = yield d
            self.assertEqual(404, resp.status_code)

            def cb(s, r):
                self.assertIsInstance(s, Session)
                self.assertIsInstance(r, Response)
                # add the parsed json data to the response
                r.data = r.json()
                return r

            d = sess.get(httpbin('get'), background_callback=cb)
            # this should block until complete
            resp = yield d
            self.assertEqual(200, resp.status_code)
            # make sure the callback was invoked
            self.assertTrue(hasattr(resp, 'data'))

            def rasing_cb(s, r):
                raise Exception('boom')

            d = sess.get(httpbin('get'), background_callback=rasing_cb)
            raised = False
            try:
                resp = yield d
            except Exception as e:
                self.assertEqual('boom', e.args[0])
                raised = True
            self.assertTrue(raised)

    def test_max_workers(self):
        """ Tests the `max_workers` shortcut. """
        from twisted.python.threadpool import ThreadPool

        with Session() as session:
            self.assertEqual(session.pool.max, 4)
        with Session(maxthreads=5) as session:
            self.assertEqual(session.pool.max, 5)
        with Session(pool=ThreadPool(maxthreads=10)) as session:
            self.assertEqual(session.pool.max, 10)
        with Session(pool=ThreadPool(maxthreads=10),
                                     maxthreads=5) as session:
            self.assertEqual(session.pool.max, 10)

    @defer.inlineCallbacks
    def test_redirect(self):
        """ Tests for the ability to cleanly handle redirects. """
        with Session() as sess:
            d = sess.get(httpbin('redirect-to?url=get'))
            resp = yield d
            self.assertIsInstance(resp, Response)
            self.assertEqual(200, resp.status_code)

            d = sess.get(httpbin('redirect-to?url=status/404'))
            resp = yield d
            self.assertEqual(404, resp.status_code)




# -*- coding: utf-8 -*-

# txrequests

"""
async requests HTTP library
~~~~~~~~~~~~~~~~~~~~~


"""

__title__ = 'txrequests'
__version__ = '0.9.6'
__build__ = 0x000000
__author__ = 'Pierre Tardy'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2013 Pierre Tardy'

# Set default logging handler to avoid "No handler found" warnings.
import logging
try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

logging.getLogger(__name__).addHandler(NullHandler())
try:
	from .sessions import Session
except ImportError:
	Session = None
__exports__ = [Session]



# -*- coding: utf-8 -*-
"""
txrequests
~~~~~~~~~~~~~~~~

This module provides a small add-on for the requests http library. It makes use
of twisted threadpool.

    from txrequests import Session

    session = Session()
    # request is run in the background
    d = session.get('http://httpbin.org/get')
    # ... do other stuff ...
    # wait for the request to complete, if it hasn't already
    @d.addCallback
    def print_res(response):
        print('response status: {0}'.format(response.status_code))
        print(response.content)

"""

from requests import Session as requestsSession
from twisted.internet import defer, reactor
from twisted.python.threadpool import ThreadPool


class Session(requestsSession):

    def __init__(self, pool=None, minthreads=1, maxthreads=4, **kwargs):
        """Creates a twisted aware Session

        Notes
        ~~~~~

        * If you provide both `pool` and `max_workers`, the latter is
          ignored and provided threadpool is used as is.
        """
        requestsSession.__init__(self, **kwargs)
        self.ownPool = False
        if pool is None:
            self.ownPool = True
            pool = ThreadPool(minthreads=minthreads, maxthreads=maxthreads)
            # unclosed ThreadPool leads to reactor hangs at shutdown
            # this is a problem in many situation, so better enforce pool stop here
            reactor.addSystemEventTrigger("after", "shutdown", lambda: pool.stop() if pool.started else None)
        self.pool = pool
        if self.ownPool:
            pool.start()

    def close(self):
        try:
            requestsSession.close(self)
        finally:
            if self.ownPool:
                self.pool.stop()

    def request(self, *args, **kwargs):
        """Maintains the existing api for Session.request.

        Used by all of the higher level methods, e.g. Session.get.

        The background_callback param allows you to do some processing on the
        response in the background, e.g. call resp.json() so that json parsing
        happens in the background thread.
        """
        def func(d):
            """Callbacks the deferred d upon request completion/error.
            """
            try:
                background_callback = kwargs.pop('background_callback', None)
                res = requestsSession.request(self, *args, **kwargs)
                if background_callback is not None:
                    res = background_callback(self, res)
                reactor.callFromThread(d.callback, res)
            except Exception as e:
                reactor.callFromThread(d.errback, e)

        d = defer.Deferred()
        self.pool.callInThread(func, d)
        return d



#!/usr/bin/env python

import os
import sys

import txrequests

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

packages = [
    'txrequests',
]

requires = [
    'requests>=1.2.0',
    'twisted>=9.0.0'
]

setup(
    name='txrequests',
    version=txrequests.__version__,
    description='Asynchronous Python HTTP for Humans.',
    long_description=open('README.rst').read(),
    author='Pierre Tardy',
    author_email='tardyp@gmail.com',
    packages=packages,
    package_dir={'txrequests': 'txrequests'},
    package_data={'txrequests': ['LICENSE', 'README.rst']},
    include_package_data=True,
    install_requires=requires,
    license='Apache License v2',
    url='https://github.com/tardyp/txrequests',
    zip_safe=False,
    classifiers=(
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ),
)
